@@ -1,18 +1,39 @@
 extern crate proc_macro;
 
 use proc_macro::TokenStream;
-use syn::{parse_macro_input, DeriveInput};
+use syn::{parse_macro_input, DeriveInput, Error};
 
+mod attributes;
 mod derives;
+mod util;
 
-#[proc_macro_derive(DeriveEntity, attributes(table))]
+#[proc_macro_derive(DeriveEntity, attributes(sea_orm))]
 pub fn derive_entity(input: TokenStream) -> TokenStream {
-    let DeriveInput { ident, attrs, .. } = parse_macro_input!(input);
+    let input = parse_macro_input!(input as DeriveInput);
+    derives::expand_derive_entity(input)
+        .unwrap_or_else(Error::into_compile_error)
+        .into()
+}
 
-    match derives::expand_derive_entity(ident, attrs) {
-        Ok(ts) => ts.into(),
-        Err(e) => e.to_compile_error().into(),
+#[proc_macro_derive(DeriveEntityModel, attributes(sea_orm))]
+pub fn derive_entity_model(input: TokenStream) -> TokenStream {
+    let input_ts = input.clone();
+    let DeriveInput {
+        ident, data, attrs, ..
+    } = parse_macro_input!(input as DeriveInput);
+
+    if ident != "Model" {
+        panic!("Struct name must be Model");
     }
+
+    let mut ts: TokenStream = derives::expand_derive_entity_model(data, attrs)
+        .unwrap_or_else(Error::into_compile_error)
+        .into();
+    ts.extend(vec![
+        derive_model(input_ts.clone()),
+        derive_active_model(input_ts),
+    ]);
+    ts
 }
 
 #[proc_macro_derive(DerivePrimaryKey)]
@@ -45,17 +66,15 @@ pub fn derive_custom_column(input: TokenStream) -> TokenStream {
     }
 }
 
-#[proc_macro_derive(DeriveModel)]
+#[proc_macro_derive(DeriveModel, attributes(sea_orm))]
 pub fn derive_model(input: TokenStream) -> TokenStream {
-    let DeriveInput { ident, data, .. } = parse_macro_input!(input);
-
-    match derives::expand_derive_model(ident, data) {
-        Ok(ts) => ts.into(),
-        Err(e) => e.to_compile_error().into(),
-    }
+    let input = parse_macro_input!(input as DeriveInput);
+    derives::expand_derive_model(input)
+        .unwrap_or_else(Error::into_compile_error)
+        .into()
 }
 
-#[proc_macro_derive(DeriveActiveModel)]
+#[proc_macro_derive(DeriveActiveModel, attributes(sea_orm))]
 pub fn derive_active_model(input: TokenStream) -> TokenStream {
     let DeriveInput { ident, data, .. } = parse_macro_input!(input);
 
@@ -85,10 +104,18 @@ pub fn derive_from_query_result(input: TokenStream) -> TokenStream {
     }
 }
 
+#[proc_macro_derive(DeriveRelation, attributes(sea_orm))]
+pub fn derive_relation(input: TokenStream) -> TokenStream {
+    let input = parse_macro_input!(input as DeriveInput);
+    derives::expand_derive_relation(input)
+        .unwrap_or_else(Error::into_compile_error)
+        .into()
+}
+
 #[doc(hidden)]
 #[proc_macro_attribute]
 pub fn test(_: TokenStream, input: TokenStream) -> TokenStream {
-    let input = syn::parse_macro_input!(input as syn::ItemFn);
+    let input = parse_macro_input!(input as syn::ItemFn);
 
     let ret = &input.sig.output;
     let name = &input.sig.ident;
